# saved as /prj/bb/crisk/guidelines_revisited/anal.hpvposnil.y2016m11d08.R
# Evaluate cumulative risk following HPV+/Pap-
# Consider non-parametric, Kaplan-Meier, various parametric fits
# Compare risks to that of ASC-US and LSIL
# Fit age as a covariate
# Programmer: Li Cheung
# Date written: Oct 23, 2016
# Last revised: March 6, 2017
# Input: hpvposnil.RData (R version of hpvposnil.sas7bdat)

rm(list=ls(all=TRUE))
library(PIMixture)

########### y2016m11d08 ################
load("anal_y2015m09d17_v2.rdata")

anal_y2015m09d17_v2$CIN3P_CI <- ifelse(is.na(anal_y2015m09d17_v2$CIN3P_CI),-999,anal_y2015m09d17_v2$CIN3P_CI)

anal_y2015m09d17_v2$CIN3P_LOWER <- ifelse(is.na(anal_y2015m09d17_v2$CIN3P_CI)==0 & anal_y2015m09d17_v2$CIN3P_CI==1,-999,anal_y2015m09d17_v2$CIN3P_LOWER)

anal_y2015m09d17_v2$CIN3P_UPPER <- ifelse(is.na(anal_y2015m09d17_v2$CIN3P_CI)==0 & anal_y2015m09d17_v2$CIN3P_CI==1,-999,ifelse(is.na(anal_y2015m09d17_v2$CIN3P_UPPER),Inf,anal_y2015m09d17_v2$CIN3P_UPPER))

hpvposnil <- subset(anal_y2015m09d17_v2,enrolhpv==1 & enrolpap==0) #HPV+/Pap-
hpvnegasc <- subset(anal_y2015m09d17_v2,enrolhpv==0 & enrolpap==1) #HPV-/ASC-US
hpvposasc <- subset(anal_y2015m09d17_v2,enrolhpv==1 & enrolpap==1) #HPV+/ASC-US
lsil <- subset(anal_y2015m09d17_v2,enrolpap==2) #LSIL

#function to obtain ASC-US risks 
#by combining risks following HPV-/ASC-US and HPV+/ASC-US
combrisk <- function(n,cr,cr.lcl,cr.ucl){
  combn <- sum(n)
  combcr <- colSums((n/combn)*cr)
  se <- -1*((log(-log(1-cr))-log(-log(1-cr.ucl)))/1.96)
  comb_se <- sqrt(colSums(((n/combn)*se)**2))
  comb_lcl <- 1-exp(-exp(log(-log(1-combcr))-1.96*comb_se))
  comb_ucl <- 1-exp(-exp(log(-log(1-combcr))+1.96*comb_se))
  return(100*as.vector(rbind(combcr,comb_lcl,comb_ucl)))
}
                                            

###Compare approaches to estimating the cumulative risk###
model <- "CIN3P_CI+CIN3P_LOWER+CIN3P_UPPER~1"
times <- seq(0,84,.1)

#non-parametric risk with confidence intervals
np <- PIMixture(p.model=model,data=hpvposnil,model="non-parametric",conf.int="TRUE")                
np_pred <- PIMixture.predict(np,data.frame(1),times)

np_hpvposasc <- PIMixture(p.model=model,data=hpvposasc,model="non-parametric")                
np_hpvposasc_pred <- PIMixture.predict(np_hpvposasc,data.frame(1),times)

np_hpvnegasc <- PIMixture(p.model=model,data=hpvnegasc,model="non-parametric")                
np_hpvnegasc_pred <- PIMixture.predict(np_hpvnegasc,data.frame(1),times)

np_lsil <- PIMixture(p.model=model,data=lsil,model="non-parametric")                
np_lsil_pred <- PIMixture.predict(np_lsil,data.frame(1),times)


#Kaplan-Meier using time of diagnosis
kmevent <- !is.na(hpvposnil$CIN3P_UPPER)
kmtime <- ifelse(kmevent,hpvposnil$CIN3P_UPPER,hpvposnil$CIN3P_LOWER)
kmtime[kmtime==0] <- .01
kmm <- survfit(Surv(kmtime,kmevent)~ 1)
#fit prevalence-incidence models
weib<-PIMixture(p.model=model,data=hpvposnil,model="logistic-Weibull")
weib_pred <- PIMixture.predict(weib,data.frame(1),times)
weib$loglikelihood

exp<-PIMixture(p.model=model,data=hpvposnil,model="logistic-exponential")
exp_pred <- PIMixture.predict(exp,data.frame(1),times)
exp$loglikelihood

lognorm<-PIMixture(p.model=model,data=hpvposnil,model="logistic-lognormal")
lognorm_pred <- PIMixture.predict(lognorm,data.frame(1),times)
lognorm$loglikelihood

loglog<-PIMixture(p.model=model,data=hpvposnil,model="logistic-loglogistic")
loglog_pred <- PIMixture.predict(loglog,data.frame(1),times)
loglog$loglikelihood

gamma<-PIMixture(p.model=model,data=hpvposnil,model="logistic-gamma")
gamma_pred <- PIMixture.predict(gamma,data.frame(1),times)
gamma$loglikelihood

gengam<-PIMixture(p.model=model,data=hpvposnil,model="logistic-gengamma")
gengam_pred <- PIMixture.predict(gengam,data.frame(1),times)
gengam$loglikelihood


#plot non-parametric without interpolation (eg. step function) across gaps
np$intmap <- np$intmap[,which(np$pf>0)]
np$pf <- np$pf[which(np$pf>0)]

for (i in 2:ncol(np$intmap)){
   begin <- np$intmap[2,(i-1)]
   end <- np$intmap[1,i]
   
   if (i==2) {
      x_coord <- seq(begin,end,by=.1)
      y_coord <- rep(cumsum(np$pf)[i-1],length(x_coord))
   } else {
      x_coord <- c(x_coord,seq(begin,end,by=.1))
      y_coord <- c(y_coord,rep(cumsum(np$pf)[i-1],length(x_coord)-length(y_coord)))
   }
}

x_coord2 <- c(15,16,21,30,57)
y_coord2 <- c(cumsum(np$pf)[2],cumsum(np$pf)[3],cumsum(np$pf)[5],cumsum(np$pf)[8],cumsum(np$pf)[14])

#create figure 1
par(mar = par()$mar + c(6, 0, 0, 0))
plot(x_coord/12,100*y_coord,type="p",lty=1,pch=20,cex=.8,xlim=c(0,7),ylim=c(0,8),main="Cumulative Risk of CIN3+ for HPV+/Pap- in a Screened Population",xlab="Years",ylab="% Cumulative Risk")
lines(x_coord2/12,100*y_coord2,type="p",pch=20,cex=.8)
lines(summary(kmm)$time/12,100*(1-summary(kmm)$surv),type="s",lty=2)
lines(times/12,100*weib_pred$CR,type="l",lty=3,lwd=1)
lines(times/12,100*gengam_pred$CR,type="l",lty=6,lwd=1)
legend(0,8,c("non-parametric","Kaplan-Meier","Logistic-Weibull","Logistic-Gen. Gamma"),cex=0.8,lty=c(1,2,3,6),lwd=c(2,1,1,1),pch=c(NA,NA,NA,NA))
mtext("% cumulative risk of cervical intraepithelial neoplasia grade 3 and cancer (CIN3+)",side=1, line=6)
mtext("following a HPV-positive/Pap-negative result at the initial screen",side=1, line=7)

### Compare risks of HPV+/Pap- against ASC-US and LSIL
weib_hpvnegasc<-PIMixture(p.model=model,data=hpvnegasc,model="logistic-Weibull")
weib_hpvnegasc_pred <- PIMixture.predict(weib_hpvnegasc,data.frame(1),times)

weib_hpvposasc<-PIMixture(p.model=model,data=hpvposasc,model="logistic-Weibull")
weib_hpvposasc_pred <- PIMixture.predict(weib_hpvposasc,data.frame(1),times)

weib_lsil<-PIMixture(p.model=model,data=lsil,model="logistic-Weibull")
weib_lsil_pred <- PIMixture.predict(weib_lsil,data.frame(1),times)

ascCR <- combrisk(n=c(nrow(hpvnegasc),nrow(hpvposasc)),
                  cr=rbind(weib_hpvnegasc_pred$CR,weib_hpvposasc_pred$CR),
                  cr.lcl=rbind(weib_hpvnegasc_pred$LL95,weib_hpvposasc_pred$LL95),
                  cr.ucl=rbind(weib_hpvnegasc_pred$UL95,weib_hpvposasc_pred$UL95))

#create figure 2
par(mar = par()$mar + c(5, 0, 0, 0))
plot(times/12,ascCR[,1],type="l",lty=1,xlim=c(0,7),ylim=c(0,8),main="Comparing CIN3+ Cumulative Risk for HPV+/Pap- to that of ASC-US and LSIL Pap",xlab="Years",ylab="% Cumulative Risk")
lines(times/12,100*weib_lsil_pred$CR,type="l",lty=2)
lines(times/12,100*weib_pred$CR,type="l",lty=3)
legend(0,8,c("ASC-US","LSIL","HPV+/Pap-"),cex=0.8,lty=c(1,2,3))
mtext("Curves showing the % cumulative risk of cervical intraepithelial neoplasia grade 3 and cancer (CIN3+)",side=1, line=6)
mtext("following (1) HPV-positive/Pap-negative cotesing results, (2) Pap result of atypical cells of",side=1, line=7)
mtext("undetermined significance (ASC-US), and (3) Pap result of low-grade squamous intraepithelial",side=1, line=8)
mtext("lesion (LSIL).",side=1, line=9)


#non-parametric risk comparison
asc_times <- sort(unique(c(np_hpvposasc$intmap[2,],np_hpvnegasc$intmap[2,])))
np_asc <- rep(NA,length(asc_times))
for (i in 1:(length(asc_times)-1)) {
  np_asc[i] <- nrow(hpvnegasc)/(nrow(hpvnegasc)+nrow(hpvposasc))*cumsum(np_hpvnegasc$pf)[max(which(np_hpvnegasc$intmap[2,]<=asc_times[i]))] +
nrow(hpvposasc)/(nrow(hpvnegasc)+nrow(hpvposasc))*cumsum(np_hpvposasc$pf)[max(which(np_hpvposasc$intmap[2,]<=asc_times[i]))]
}

#plot non-parametric risk curves
plot(x_coord/12,100*y_coord,type="p",lty=1,pch=20,cex=.8,xlim=c(0,7),ylim=c(0,8),main="Comparing CIN3+ Cumulative Risk for HPV+/Pap- to that of ASC-US and LSIL PAP",xlab="Years",ylab="% Cumulative Risk")
lines(x_coord2/12,100*y_coord2,type="p",pch=20,cex=.8)
lines(asc_times,100*np_asc)
lines(np_lsil$intmap[2,],100*cumsum(np_lsil$pf))


#consider HPV+/Pap- age-specific risks
hpvposnil$AGE_CAT <- ifelse(as.numeric(hpvposnil$ENRL_AGE)<35,1,ifelse(as.numeric(hpvposnil$ENRL_AGE)<40,2,ifelse(as.numeric(hpvposnil$ENRL_AGE)<45,3,ifelse(as.numeric(hpvposnil$ENRL_AGE)<50,4,ifelse(as.numeric(hpvposnil$ENRL_AGE)<55,5,ifelse(as.numeric(hpvposnil$ENRL_AGE)<60,6,7))))))

hpvposnil$AGE_CAT2 <- ifelse(as.numeric(hpvposnil$ENRL_AGE)>=35 & as.numeric(hpvposnil$ENRL_AGE)<40,1,0)
hpvposnil$AGE_CAT3 <- ifelse(as.numeric(hpvposnil$ENRL_AGE)>=40 & as.numeric(hpvposnil$ENRL_AGE)<45,1,0)
hpvposnil$AGE_CAT4 <- ifelse(as.numeric(hpvposnil$ENRL_AGE)>=45 & as.numeric(hpvposnil$ENRL_AGE)<50,1,0)
hpvposnil$AGE_CAT5 <- ifelse(as.numeric(hpvposnil$ENRL_AGE)>=50 & as.numeric(hpvposnil$ENRL_AGE)<55,1,0)
hpvposnil$AGE_CAT6 <- ifelse(as.numeric(hpvposnil$ENRL_AGE)>=55,1,0)
p.data <- data.frame(c(1,0,0,0,0,0),c(0,1,0,0,0,0),c(0,0,1,0,0,0),c(0,0,0,1,0,0),c(0,0,0,0,1,0),c(0,0,0,0,0,1))
colnames(p.data) <- c("AGE_CAT1","AGE_CAT2","AGE_CAT3","AGE_CAT4","AGE_CAT5","AGE_CAT6")
model <- "CIN3P_CI+CIN3P_LOWER+CIN3P_UPPER~AGE_CAT2+AGE_CAT3+AGE_CAT4+AGE_CAT5+AGE_CAT6"
weib_age<-PIMixture(p.model=model,data=hpvposnil,model="logistic-Weibull")
times <- c(0,12,24,36,48,60,72,84)
weib_age_pred <- PIMixture.predict(weib_age,p.data,times)


#plot age-specific risks and compare to ASC-US and LSIL
plot(times/12,100*ascCR,type="l",lty=1,xlim=c(0,7),ylim=c(0,9),main="Comparing CIN3+ Cumulative Risk for HPV+/NILM to that of ASC-US and LSIL Pap",xlab="Years",ylab="% Cumulative Risk")
lines(times/12,100*weib_lsil_pred$CR,type="l",lty=2)
lines(times/12,100*weib_age_pred$CR[weib_age_pred$prev.predictor=="1,0,0,0,0,0"],type="l",lty=3,col="Blue")
lines(times/12,100*weib_age_pred$CR[weib_age_pred$prev.predictor=="1,1,0,0,0,0"],type="l",lty=3,col="Red")
lines(times/12,100*weib_age_pred$CR[weib_age_pred$prev.predictor=="1,0,1,0,0,0"],type="l",lty=3,col="Green")
lines(times/12,100*weib_age_pred$CR[weib_age_pred$prev.predictor=="1,0,0,1,0,0"],type="l",lty=3,col="Purple")
lines(times/12,100*weib_age_pred$CR[weib_age_pred$prev.predictor=="1,0,0,0,1,0"],type="l",lty=3,col="Pink")
lines(times/12,100*weib_age_pred$CR[weib_age_pred$prev.predictor=="1,0,0,0,0,1"],type="l",lty=3,col="Orange")
legend(0,9,c("ASC-US","LSIL","HPV+/Pap- Age 30-34","HPV+/Pap- Age 35-39","HPV+/Pap- Age 40-44","HPV+/Pap- Age 45-49","HPV+/Pap- Age 50-54","HPV+/Pap- Age 55-64"),cex=0.8,lty=c(1,2,3,3,3,3,3,3),col=c("Black","Black","Blue","Red","Green","Purple","Pink","Orange"))

